#!/usr/bin/env python
#
# $Id$
#  
# convert files in a directory to my way of naming things, recording track
# numbers to gen a playlist, writing id3 tags, etc.
#
# Started 5/8/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

import os, sys
import re

META_MAP = {
    "a": "artist",
    "m": "multi_artist",    
    "t": "title",
    "A": "album",
    "n": "track"
    }

omnipotent_meta = {} # information which is applicable to all files we're
                     # processing
   

def get_output(format, meta, file):
    """ construct output string from format and meta info """

    index = 1

    result = format[:]

    for (k,v) in META_MAP.items():
        if result.find("%"+k) != -1:
            if not meta.has_key(v):
                print file
                meta[v] = raw_input(v + ": ")
                if k=="a":
                    omnipotent_meta[v]=meta[v]

            result = result.replace("%"+k, meta[v])

    return result

    
def extract_meta(format, string):
    """ convert %a, %t etc and store the meta information we can. """

    result = {}
    for (k,v) in omnipotent_meta.items():
        result[k]=v
    
    f = format[:]

    f = re.escape(f)

    f = f.replace("\\%a", r"(.+)")
    f = f.replace("\\%n", r"(\d+)")
    f = f.replace("\\%A", r"(.+)")
    f = f.replace("\\%t", r"(.+)")

    m = re.search(f, string)

    if m == None:
        # match failed.
        return {}

    newstr = format[:]

    for match in m.groups():
        index = newstr.find("%")

        if index == -1:
            # more matches than meta chars, oh dear
            print "shit."
            return

        meta = newstr[index+1:index+2]
        newstr = newstr[index+2:]

        if meta == "a": # artist
            result['artist'] = match

        elif meta == "t": # title
            result['title'] = match

        elif meta == "n": # track
            result['track'] = match

        elif meta == "A": # album
            result['album'] = match            
            

    return result
        
        
    
if __name__ == "__main__":

    files = os.listdir(sys.argv[1:2] or ".")
    
    if not files:
        print "No files in this directory, aborting"
        sys.exit(1)
	
	
    print "Here's the files you've selected:\n"
    for x in files: print x

    print
    print "Input format (minus ext):\n " + repr(META_MAP)
    in_format = raw_input("> ")
    in_format += ".mp3"

    print
    print "Output format (minus ext) [%a - %t]: "
    out_format = raw_input("> ")
    print
    
    if not out_format:
        out_format = "%a - %t.mp3"
    else:
        out_format += ".mp3"
        

    playlist = []
    for file in files:
        # collect what we can from the file
        meta = extract_meta(in_format, file)

        out = get_output(out_format, meta, file)
        
        playlist.append(out.lower())

        print out.lower()


    print
    ans = raw_input("Rename? [y/N] ")

    if ans.find('y')==0:

        for x in range(len(files)):

            if os.path.isfile(playlist[x]):
                # if the dest file exists, don't rename
                print "Dest file %s exists, not renaming" % \
                      (playlist[x])
                continue
                    
            os.rename(files[x], playlist[x])

        print "Cur dir: " + os.path.abspath(".")
        ans = raw_input("Playlist name (minus extention): ")

        try:
            pl_file = open(ans + ".m3u", "w")
        except OSError:
            print "Couldn't open playlist file, aborting! Here's the playlist:"
            print repr(playlist)

        for x in playlist:
            pl_file.write(x + '\n')
            
    
        

    
    


