#!/usr/bin/env python
#
# $Id: fileops.py,v 1.1 2001/09/03 09:36:21 resolve Exp resolve $
#
# fileops.py - handle iterating through a directory looking for mp3s to add
#              to a playlist, etc. ties a few modules together
#
# started 24/8/01 by Damien Elmes <resolve@repose.cx>
#

import os
import stat

import player
import playlist

import ui

from debug import * # fix

def add_uri_to_playlist(uri):
    """ determine if we've been passed a file, directory, playlist, etc """

    if uri.find(".m3u", -4) > 0:        #FIXME case insensitive
        add_m3u_to_playlist(uri)
# this behaviour isn't logical - maybe in the rsync sense :-)
#    elif uri.find("/", -1) > 0:
        # trailing /, add dir non-recursive
#        add_directory_to_playlist(uri)
    elif os.path.isdir(uri):
        add_directory_recursively_to_playlist(uri)
    else:
        add_file_to_playlist(uri)

def add_m3u_to_playlist(file):
    """ read a .m3u specifying a list of mp3s to play """

    f = open(file).readlines()

    i = 1
    for x in f:
        x = x.strip()
        add_file_to_playlist(os.path.join(os.path.dirname(file), x),\
                             {'playlist': file, 'track': i})

        i += 1

def add_directory_recursively_to_playlist(dir):
    """ recurse through and add all files that have a handler """

    dirs = []
    files = []
    
    for ent in os.listdir(dir):
        ent = os.path.join(dir, ent)

        if os.path.isdir(ent):
            # if we have a directory, note it
            dirs.append(ent)
        else:
            # it's not a directory, so we try and read the file
            files.append(ent)

    # now we recurse in
    for ent in dirs:
        add_directory_recursively_to_playlist(ent)

    have_m3u = 0
    for ent in files:
        if ent.find(".m3u")==len(ent)-4:
            have_m3u = 1
            add_m3u_to_playlist(ent)

    # if there are m3u files in the directory, use them instead of the files
    if not have_m3u:
        for ent in files:
            add_file_to_playlist(ent)
            

def add_directory_to_playlist(dir):
    """ add all the files in a directory to a playlist, checking to make sure
    file has a handler for it (thus ELF files etc should not appear) """


    # note: function and method calls are expensive in python. we duplicate
    # the add_file functionality here in order to speed things up.
    files = os.listdir(dir)

    ui.notice("adding %d files in directory %s to playlist..." % \
              (len(files), os.path.basename(dir)))
    ui.refresh()

    for x in files:
        path = os.path.join(dir, x)
        add_file_to_playlist(path)

def add_file_to_playlist(path, extra=None):
    """ add a single file to the playlist if we have a handler for it """

    try:
        x = os.stat(path)
        if not stat.S_IREAD & x[0]:
            debug(3, "fileops: dropping " + str(path))
            return
    except OSError:
        return
    
    if not player.have_handler(path):
        debug(1, "dropping unhandled file type: " + \
              os.path.basename(path))
        return

    t = player.filename_to_meta(path)

    dict = {
        'file':     os.path.basename(path),
        'dir':      os.path.dirname(path),
        'path':     path,
        'artist':   t['artist'],
        'title':    t['title'],
        }

    if extra:
        for (k,v) in extra.items():
            dict[k]=v

    playlist.list.append(dict)


class traversePlaylist:
    """ iterates over a playlist, adding time information to the list. """
    pass



    
            

    

