#!/usr/bin/env python
#
# $Id: mp3meta.py,v 1.1 2001/09/03 09:04:08 resolve Exp resolve $
#  
# Module used for manipulating meta information on mp3s - extracting and writing
# id3 tags, determining artist and title from the filename, etc.
#
# Started 6/7/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#


import sys, os, re
import stat
import md5
import mp3
import mp3id3

def nice_case(str):
    """capitalise each word of the string that's passed, and return new"""

    keep_lower = [ "and", "or", "the", "of", "a", "ii", "iii" ]

    a = []
    for w in str.split():
        if w in keep_lower:
            a.append(w)
        else:
            a.append(w.capitalize())
            
    return " ".join(a)

def parse_filename(filename):
    """try and discover some meta information such as artist, title etc
    by our filename"""

    # don't clobber
    f = filename[:]

    # lop the filename extention off
    f = re.sub('\.[mM][pP][23]', "", f)

    # try and ween out artist - title
    m = re.match("^[[{(]?((?:\S *)+?) ?[]\-_\.)}] *((?:\S *)+?)$", f)

    if not m:
        # oh dear - we couldn't parse the filename.
        return None

    return {'artist': m.group(1).strip(), 'title': m.group(2).strip()}

def parse_id3(path):
    """ grab what we can out of of the id3 """

    return mp3id3.parse(path)

class file:
    """Contains all the useful information about a file we're concerned with"""

    def __init__(self, path=""):
        """called with either the path of an mp3, or nothing."""
        # this is the standard way to proceed
        if path: self.set_path(path)

    def set_path(self, path):
        """ take a filename and update all the internal information so
        that it corresponds to a new file"""

        self.path = path
        self.file = os.path.basename(path)

    def parse_other(self):
        """ make available information like file size, md5 """
        
        # filesize
        self.size = os.stat(self.path)[stat.ST_SIZE]

        # md5 sum
        f = open(self.path)
        m = md5.new()

        # read up to the first 512K of the file, and checksum it.
        m.update(f.read(512 * 1024))
        self.md5sum = m.hexdigest()

    def parse_content(self):
        """ stuff which needs to actually open the file, and is thus slow.
        id3 tags are included here """

        # now the mp3/frame info
        t = mp3.mp3info(self.path)

        if not t:
            # if we can't parse the mp3 data, then it's not a valid mp3 file
            return 0
        
        self.mins = t['MM']
        self.secs = t['SS']

        self.length = self.mins * 60 + self.secs
        self.bitrate = t['BITRATE']

        

        
                                        
        

        

        

        
        

    
            

        
