#!/usr/bin/env python
#
# $Id: orbital.py,v 1.1 2001/10/29 07:03:39 resolve Exp resolve $
#
# Started 5/8/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

"""
Interface to a Matrix Orbital screen and optional keypad
"""

import os, sys
import atexit
import termios
if sys.version_info[0] >= 2 and sys.version_info[1] >= 1:
    import termios as TERMIOS
else:
    import TERMIOS
 

import multiplex

ESC = chr(0xFE)

CUSTOM_CHAR_LIMIT=8

class new:
    """ create a new orbital connection """

    def _onexit(self):
        """ called by atexit, clean up the screen and prevent further
        modifications """

        # reset serial
        termios.tcsetattr(self.screen_fd, TERMIOS.TCSANOW, self.old_termios)

    def __init__(self, device, baud=19200):

        baud = "B" + str(baud)

        # FIXME: raise orbital.error
        self.screen_fd = os.open(device, os.O_RDWR | os.O_NOCTTY)

        atexit.register(self._onexit)
        
        # set speed
        self.old_termios = termios.tcgetattr(self.screen_fd)

        old = self.old_termios[:]

        # control flags
        old[2] |= TERMIOS.CS8 | TERMIOS.CLOCAL

        # iflags: don't want terminal echo etc
        old[3] &= ~(TERMIOS.ECHO | TERMIOS.ECHONL | \
                    TERMIOS.ICANON | TERMIOS.ISIG)
        
        #old[2] &= ~(TERMIOS.CSIZE|TERMIOS.PARENB|TERMIOS.HUPCL);

        old[1] &= ~TERMIOS.OPOST;

        #old[0] &= ~(TERMIOS.IGNBRK|TERMIOS.BRKINT|TERMIOS.PARMRK| \
        #            TERMIOS.ISTRIP|TERMIOS.INLCR|TERMIOS.IGNCR|TERMIOS.ICRNL|\
        #            TERMIOS.IXON);

        old[4] = getattr(TERMIOS, baud) # ispeed
        old[5] = getattr(TERMIOS, baud) # ospeed

        termios.tcsetattr(self.screen_fd, TERMIOS.TCSANOW, old)

        # set up a hook for incoming data
        mask = multiplex.POLLIN
        multiplex.add_watch(self.screen_fd, mask, self.on_keypad)

        self.keypad_cb = None

    def on_keypad(self, fd):
        if self.keypad_cb != None:
            self.keypad_cb(fd)

    def set_keypad_cb(self, cb):
        self.keypad_cb = cb
    

    def write(self, msg, x=None, y=None, size=None, pad=None):
        """ write a message to the screen

        x and y coods are optional, as is size, which the message will be
        chopped to. """

        if x != None and y != None:
            self.set_pos(x, y)

        if size != None:
            msg = msg[:size]

        if pad != None and size!=None:
            msg += " " * (size-len(msg))

        os.write(self.screen_fd, msg)

    def clear_screen(self):
        self.write(ESC + 'X')

    def set_pos(self, x, y):
        """ move the cursor to (x, y) """

        self.write(ESC + 'G' + chr(x) + chr(y))

    def set_startup_screen(self, msg):
        """ set the new screen on startup """

        # crop or pad
        if len(msg) < 80:
            msg += " " * (80-len(msg))
        elif len(msg)>80:
            msg = msg[:80]

        self.write(ESC + '@' + msg)

    def set_contrast(self, contrast):
        if 0 > contrast > 255:
            raise TypeError, "Contrast must be between 0-255!"

        self.write(ESC + 'P' + chr(contrast))

    def backlight_on(self):
        self.write(ESC + 'B')

    def backlight_off(self):
        self.write(ESC + 'F')

    def set_custom_char(self, index, bytes):
        """ write a custom char at index (from 0-7), using a seq of 8 bytes

        the bytes only cares about the lower 5 bits - please see orbital
        documentation """

        if index > CUSTOM_CHAR_LIMIT+1:
            raise ValueError, "Index exceeds the number of custom chars screen"\
                  " can store"

        if len(bytes) != 8:
            raise ValueError, "Bytestring should be exactally 8 bytes"

        self.write(ESC + 'N' + chr(index))
        for x in bytes:
            self.write(chr(x))

    def clear_keybuffer(self):
        """ flush the key buffer, discarding output """
        self.write(ESC + 'E')

            
        
        
        

    
        

    
