#!/usr/bin/env python
#
# $Id$
#  
# control muse via writing/reading to a unix socket.
#
# Started 5/8/01 by Damien Elmes <resolve@repose.cx>
# This file is licensed under the GPL.
#

import os
import socket

import multiplex
import ui
import config
import control

from debug import debug

PLUGIN_TYPE="ui"

config_defaults = {
    "ctrl_file": os.path.expanduser("~/.muse/socket")
    }

config.set_defaults("ui_remote_unix", config_defaults)


class new(ui.plugin):
    """ provides an abstract class for a interface plugin, for muse"""

    def __init__(self):

        ui.plugin.__init__(self)

        self.ctrl_file = config.get("ui_remote_unix", "ctrl_file")
        self.conns = {}

        try: 
            os.unlink(self.ctrl_file) # FIXME is this the correct way to go?
        except OSError: pass
        
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

        self.sock.setblocking(0)

        self.sock.bind(self.ctrl_file)

        self.sock.listen(1)

        # good, we're non blocking now. set up a hook for accept calls

        multiplex.add_watch(self.sock.fileno(), multiplex.POLLIN,
                            self.on_accept)

    def __del__(self):
        pass

    def on_accept(self, fd):
        debug(3, __name__ + ": new client")
        try:
            x = self.sock.accept()
        except None:
            # shouldn't happen
            return

        sock = x[0]

        self.conns[sock.fileno()] = sock

        multiplex.add_watch(sock.fileno(), multiplex.POLLIN |
                            multiplex.POLLHUP, self.on_recv)

    def on_recv(self, fd):
        """ Called when there is action on a socket """

        try:
            res = self.conns[fd].recv(256)
        except socket.error:
            res = None

        if not res:
            # no more data to read, clean up this socket.
            multiplex.del_watch(fd)
            self.conns[fd].close()
            del self.conns[fd]
            return

        out = str(control.cmd(res))
        try:
            self.conns[fd].send(out.strip())
            self.conns[fd].send("\n")            
        except socket.error:
            debug(3, "socket error on ui_remote_unix output")
            return
            

    def notice(self, msg): pass


if __name__ == "__main__":

    f = new()


        
        


    
    
        
        

    
        
    
